#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.21 19:00:00                  #
# ================================================== #

import re

from PySide6.QtCore import Qt, QObject, Signal
from PySide6.QtWebEngineCore import QWebEngineSettings, QWebEnginePage
from PySide6.QtWebEngineWidgets import QWebEngineView
from PySide6.QtGui import QAction, QIcon, QKeySequence

from pygpt_net.core.text.web_finder import WebFinder
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class ChatWebOutput(QWebEngineView):
    def __init__(self, window=None):
        """
        HTML output (Webkit)

        :param window: main window
        """
        super(ChatWebOutput, self).__init__(window)
        self.window = window
        self.finder = WebFinder(window, self)
        self.loadFinished.connect(self.on_page_loaded)
        self.customContextMenuRequested.connect(self.on_context_menu)
        self.signals = WebEngineSignals()
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.plain = ""
        self.html_content = ""

    def set_plaintext(self, text: str):
        self.plain = text

    def set_html_content(self, html: str):
        self.html_content = "<html>" + html + "</html>"

    def on_context_menu(self, position):
        """
        Context menu event

        :param position: position
        """
        menu = self.createStandardContextMenu()

        # remove defaults
        for action in menu.actions()[::-1]:
            menu.removeAction(action)

        selected_text = ""
        is_selection = self.page().hasSelection()
        if is_selection:
            selected_text = self.get_selected_text()

        if is_selection:
            # copy
            action = QAction(QIcon(":/icons/copy.svg"), trans('action.copy'), self)
            action.triggered.connect(self.copy_selected_text)
            menu.addAction(action)

            # audio read
            action = QAction(QIcon(":/icons/volume.svg"), trans('text.context_menu.audio.read'), self)
            action.triggered.connect(
                lambda: self.signals.audio_read.emit(selected_text)
            )
            menu.addAction(action)

            # copy to
            copy_to_menu = self.window.ui.context_menu.get_copy_to_menu(self, selected_text)
            menu.addMenu(copy_to_menu)

            # save as (selected)
            action = QAction(QIcon(":/icons/save.svg"), trans('action.save_selection_as'), self)
            action.triggered.connect(
                lambda: self.signals.save_as.emit(selected_text, 'txt')
            )
            menu.addAction(action)
        else:
            # select all
            action = QAction(QIcon(":/icons/copy.svg"), trans('action.select_all'), self)
            action.triggered.connect(self.select_all_text)
            menu.addAction(action)

            # save as (all) - plain
            action = QAction(QIcon(":/icons/save.svg"), trans('action.save_as') + " (text)", self)
            action.triggered.connect(
                lambda: self.signals.save_as.emit(re.sub(r'\n{2,}', '\n\n', self.plain), 'txt')
            )
            menu.addAction(action)

            # save as (all) - html
            action = QAction(QIcon(":/icons/save.svg"), trans('action.save_as') + " (html)", self)
            action.triggered.connect(
                lambda: self.signals.save_as.emit(re.sub(r'\n{2,}', '\n\n', self.html_content), 'html')
            )
            menu.addAction(action)

        action = QAction(QIcon(":/icons/search.svg"), trans('text.context_menu.find'), self)
        action.triggered.connect(self.find_open)
        action.setShortcut(QKeySequence("Ctrl+F"))
        menu.addAction(action)

        menu.exec_(self.mapToGlobal(position))

    def update_zoom(self):
        """Update zoom from config"""
        if self.window.core.config.has("zoom"):
            self.page().setZoomFactor(self.window.core.config.get("zoom"))

    def get_zoom_value(self) -> float:
        """
        Get zoom value

        :return: zoom value
        """
        return self.page().zoomFactor()

    def reset_current_content(self):
        """Reset current content"""
        self.plain = ""
        self.html_content = ""

    def update_current_content(self):
        """Update current content"""
        self.page().runJavaScript("document.getElementById('container').outerHTML", 0, self.set_plaintext)
        self.page().runJavaScript("document.documentElement.innerHTML", 0, self.set_html_content)

    def on_page_loaded(self, success):
        """
        On page loaded

        :param success: True if loaded successfully
        """
        if success:
            self.window.controller.chat.render.on_page_loaded()

    def get_selected_text(self) -> str:
        """
        Get selected text

        :return: selected text
        """
        return self.page().selectedText()

    def copy_selected_text(self):
        """Copy selected text"""
        self.page().triggerAction(QWebEnginePage.Copy)

    def select_all_text(self):
        """Select all text"""
        self.page().triggerAction(QWebEnginePage.SelectAll)

    def unselect_text(self):
        """Unselect text"""
        self.page().triggerAction(QWebEnginePage.Unselect)

    def find_open(self):
        """Open find dialog"""
        self.window.controller.finder.open(self.finder)

    def on_update(self):
        """On content update"""
        self.finder.clear()  # clear finder

    def focusInEvent(self, e):
        """
        Focus in event

        :param e: focus event
        """
        super(ChatWebOutput, self).focusInEvent(e)
        self.window.controller.finder.focus_in(self.finder)


class CustomWebEnginePage(QWebEnginePage):
    """Custom WebEnginePage to handle web events"""
    def __init__(self, window, parent):
        super(CustomWebEnginePage, self).__init__(window)
        self.window = window
        self.parent = parent
        self.findTextFinished.connect(self.on_find_finished)
        self.contentsSizeChanged.connect(self.on_view_changed)
        self.selectionChanged.connect(self.on_selection_changed)
        self.settings().setAttribute(
            QWebEngineSettings.LocalContentCanAccessFileUrls, True
        )
        self.settings().setFontFamily(QWebEngineSettings.StandardFont, 'Lato')
        self.settings().setFontFamily(QWebEngineSettings.FixedFont, 'Monaspace Neon')
        self.settings().setFontFamily(QWebEngineSettings.SerifFont, 'Monaspace Neon')
        if self.window.core.config.has("zoom"):
            self.setZoomFactor(self.window.core.config.get("zoom"))

    def on_find_finished(self, result):
        """
        On find text finished

        :param result: Find result
        """
        current = int(result.activeMatch())
        num = int(result.numberOfMatches())
        self.parent.finder.current_match_index = current
        self.parent.finder.matches = num
        self.parent.finder.on_find_finished()

    def on_view_changed(self):
        """On view changed"""
        zoom = self.zoomFactor()
        self.window.core.config.set("zoom", zoom)
        option = self.window.controller.settings.editor.get_option('zoom')
        option['value'] = zoom
        self.window.controller.config.apply(
            parent_id='config',
            key='zoom',
            option=option,
        )

    def on_selection_changed(self):
        """On selection changed"""
        pass

    def acceptNavigationRequest(self, url,  _type, isMainFrame):
        """
        On navigation (link click) event

        :param url: URL
        :param _type: Navigation type
        :param isMainFrame: True if main frame
        """
        if _type == QWebEnginePage.NavigationTypeLinkClicked:
            self.window.core.filesystem.url.handle(url)
            return False
        return super().acceptNavigationRequest(url,  _type, isMainFrame)


class WebEngineSignals(QObject):
    save_as = Signal(str, str)
    audio_read = Signal(str)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.21 19:00:00                  #
# ================================================== #

from PySide6.QtCore import Slot, QTimer
from bs4 import BeautifulSoup

from pygpt_net.core.render.base import BaseRenderer
from pygpt_net.core.render.markdown.renderer import Renderer as MarkdownRenderer
from pygpt_net.core.render.plain.renderer import Renderer as PlainTextRenderer
from pygpt_net.core.render.web.renderer import Renderer as WebRenderer
from pygpt_net.item.ctx import CtxItem


class Render:
    def __init__(self, window=None):
        """
        Render controller

        :param window: Window instance
        """
        self.window = window
        self.markdown_renderer = MarkdownRenderer(window)
        self.plaintext_renderer = PlainTextRenderer(window)
        self.web_renderer = WebRenderer(window)
        self.engine = None

    def setup(self):
        """Setup render"""
        self.engine = self.window.core.config.get("render.engine")
        if self.engine == "web":
            self.connect_signals()

    def connect_signals(self):
        """Connect signals"""
        signals = self.web_renderer.get_output_node().signals
        signals.save_as.connect(self.handle_save_as)
        signals.audio_read.connect(self.handle_audio_read)

    def get_engine(self):
        """Get current engine"""
        return self.engine

    def get_renderer(self) -> BaseRenderer:
        """
        Get current renderer instance

        :return: Renderer instance
        """
        # get selected renderer
        if self.window.core.config.get('render.plain'):
            return self.plaintext_renderer
        else:
            if self.engine == "web":
                return self.web_renderer
            else:
                return self.markdown_renderer

    def switch(self):
        """Switch renderer"""
        plain = self.window.core.config.get('render.plain')
        if plain:
            self.window.controller.theme.markdown.clear()
            if self.window.ui.nodes['output'] is not None:
                if self.window.ui.nodes['output'].isVisible():
                    self.window.ui.nodes['output'].setVisible(False)
                    self.window.ui.nodes['output_plain'].setVisible(True)
        else:
            self.window.controller.ctx.refresh()
            self.window.controller.theme.markdown.update(force=True)
            if self.window.ui.nodes['output_plain'].isVisible():
                if self.window.ui.nodes['output'] is not None:
                    self.window.ui.nodes['output'].setVisible(True)
                    self.window.ui.nodes['output_plain'].setVisible(False)

    def begin(self, stream: bool = False):
        """
        Render begin

        :param stream: True if it is a stream
        """
        self.get_renderer().begin(stream)
        self.update()

    def end(self, stream: bool = False):
        """
        Render end

        :param stream: True if it is a stream
        """
        self.get_renderer().end(stream)
        self.update()

    def end_extra(self, stream: bool = False):
        """
        Render end extra

        :param stream: True if it is a stream
        """
        self.get_renderer().end_extra(stream)
        self.update()

    def stream_begin(self):
        """Render stream begin"""
        self.get_renderer().stream_begin()
        self.update()

    def stream_end(self):
        """Render stream end"""
        self.get_renderer().stream_end()
        self.update()

    def clear_output(self):
        """Clear output"""
        self.get_renderer().clear_output()
        self.update()

    def clear_input(self):
        """Clear input"""
        self.get_renderer().clear_input()

    def reset(self):
        """Reset"""
        self.get_renderer().reset()
        self.update()

    def reload(self):
        """Reload output"""
        self.get_renderer().reload()
        self.update()

    def append_context(self, items: list, clear: bool = True):
        """
        Append all context to output

        :param items: Context items
        :param clear: True if clear all output before append
        """
        self.get_renderer().append_context(items, clear)
        self.update()

    def append_input(self, item: CtxItem):
        """
        Append text input to output

        :param item: context item
        """
        self.get_renderer().append_input(item)
        self.update()

    def append_output(self, item: CtxItem):
        """
        Append text output to output

        :param item: context item
        """
        self.get_renderer().append_output(item)
        self.update()

    def append_extra(self, item: CtxItem, footer: bool = False):
        """
        Append extra data (images, files, etc.) to output

        :param item: context item
        :param footer: True if it is a footer
        """
        self.get_renderer().append_extra(item, footer)
        self.update()

    def append_chunk(self, item: CtxItem, text_chunk: str, begin: bool = False):
        """
        Append output stream chunk to output

        :param item: context item
        :param text_chunk: text chunk
        :param begin: if it is the beginning of the stream
        """
        self.get_renderer().append_chunk(item, text_chunk, begin)
        self.update()

    def on_page_loaded(self):
        """On page loaded callback"""
        self.get_renderer().on_page_loaded()
        self.update()

    def update(self):
        """On update"""
        self.window.ui.nodes['output'].on_update()

    def clear_all(self):
        """Clear all"""
        self.get_renderer().clear_all()
        self.update()

    @Slot(str, str)
    def handle_save_as(self, text: str, type: str = 'txt'):
        """
        Handle save as signal

        :param text: Text to save
        :param type: File type
        """
        if type == 'html':
            text = self.pretify_html(text)
        else:
            text = self.strip_html(text)
        # fix: QTimer required here to prevent crash if signal emitted from WebEngine window
        QTimer.singleShot(0, lambda: self.window.controller.chat.common.save_text(text, type))

    @Slot(str)
    def handle_audio_read(self, text: str):
        """
        Handle audio read signal

        :param text: Text to read
        """
        self.window.controller.audio.read_text(text)

    def strip_html(self, html: str) -> str:
        """
        Strip HTML tags and get plain text

        :param html: HTML content
        :return: Plain text
        """
        if html == "":
            return ""
        try:
            soup = BeautifulSoup(html, 'html.parser')
            # remove headers from code blocks
            for tag in soup.find_all('p', class_='code-header-wrapper'):
                empty = soup.new_tag('p')
                empty.string = '\n'
                tag.replace_with(empty)
            # add separators
            for tag in soup.find_all('div', class_='msg-bot'):
                sep = soup.new_tag('p')
                sep.string = '\n\n'
                tag.insert_before(sep)
            for tag in soup.find_all('div', class_='msg-user'):
                sep = soup.new_tag('p')
                sep.string = '\n\n'
                tag.insert_before(sep)
            return soup.get_text()
        except Exception as e:
            pass
        return ""

    def pretify_html(self, html: str) -> str:
        """
        Pretify HTML content

        :param html: HTML content
        :return: HTML content
        """
        try:
            soup = BeautifulSoup(html, 'html.parser')
            # remove copy from code blocks
            for tag in soup.find_all('a', class_='code-header-copy'):
                tag.decompose()
            # remove action icons
            for tag in soup.find_all('div', class_='action-icons'):
                tag.decompose()
            return str(soup)
        except Exception as e:
            pass
        return html



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.10 23:00:00                  #
# ================================================== #

from pygpt_net.ui.dialog.about import About
from pygpt_net.ui.dialog.applog import AppLog
from pygpt_net.ui.dialog.assistant import Assistant
from pygpt_net.ui.dialog.changelog import Changelog
from pygpt_net.ui.dialog.create import Create
from pygpt_net.ui.dialog.db import Database
from pygpt_net.ui.dialog.debug import Debug
from pygpt_net.ui.dialog.dictionary import Dictionary
from pygpt_net.ui.dialog.editor import Editor
from pygpt_net.ui.dialog.find import Find
from pygpt_net.ui.dialog.image import Image
from pygpt_net.ui.dialog.license import License
from pygpt_net.ui.dialog.logger import Logger
from pygpt_net.ui.dialog.models import Models
from pygpt_net.ui.dialog.plugins import Plugins
from pygpt_net.ui.dialog.preset import Preset
from pygpt_net.ui.dialog.preset_plugins import PresetPlugins
from pygpt_net.ui.dialog.profile import Profile, ProfileEdit
from pygpt_net.ui.dialog.rename import Rename
from pygpt_net.ui.dialog.settings import Settings
from pygpt_net.ui.dialog.snap import Snap
from pygpt_net.ui.dialog.start import Start
from pygpt_net.ui.dialog.update import Update
from pygpt_net.ui.dialog.workdir import Workdir
from pygpt_net.ui.widget.dialog.alert import AlertDialog
from pygpt_net.ui.widget.dialog.confirm import ConfirmDialog

class Dialogs:
    def __init__(self, window=None):
        """
        Dialogs setup

        :param window: Window instance
        """
        self.window = window
        self.about = About(self.window)
        self.assistant = Assistant(self.window)
        self.app_log = AppLog(self.window)
        self.changelog = Changelog(self.window)
        self.create = Create(self.window)
        self.database = Database(self.window)
        self.debug = Debug(self.window)
        self.dictionary = Dictionary(self.window)
        self.editor = Editor(self.window)
        self.find = Find(self.window)
        self.image = Image(self.window)
        self.license = License(self.window)
        self.logger = Logger(self.window)
        self.preset = Preset(self.window)
        self.profile_item = ProfileEdit(self.window)
        self.rename = Rename(self.window)
        self.snap = Snap(self.window)
        self.start = Start(self.window)
        self.update = Update(self.window)
        self.workdir = Workdir(self.window)

    def setup(self):
        """Setup dialogs"""
        for id in self.window.controller.dialogs.debug.ids:
            self.debug.setup(id)

        self.about.setup()
        self.app_log.setup()
        self.changelog.setup()
        self.create.setup()
        self.preset.setup()
        self.assistant.setup()
        self.editor.setup()
        self.find.setup()
        self.rename.setup()
        self.snap.setup()
        self.start.setup()
        self.update.setup()
        self.image.setup()
        self.license.setup()
        self.logger.setup()
        self.database.setup()
        self.workdir.setup()
        self.profile_item.setup()

        self.window.settings = Settings(self.window)
        self.window.profiles = Profile(self.window)
        self.window.plugin_settings = Plugins(self.window)
        self.window.plugin_presets = PresetPlugins(self.window)
        self.window.model_settings = Models(self.window)

        self.window.ui.dialog['alert'] = AlertDialog(self.window)
        self.window.ui.dialog['confirm'] = ConfirmDialog(self.window)

    def post_setup(self):
        """Post setup dialogs (after plugins and rest of data is registered)"""
        self.dictionary.setup()

    def confirm(
            self,
            type: str,
            id: any,
            msg: str,
            parent_object=None
    ):
        """
        Show confirm dialog

        :param type: confirm type
        :param id: confirm object id
        :param msg: message to show
        :param parent_object: parent object
        """
        self.window.ui.dialog['confirm'].type = type
        self.window.ui.dialog['confirm'].id = id
        self.window.ui.dialog['confirm'].message.setText(msg)
        self.window.ui.dialog['confirm'].parent_object = parent_object
        self.window.ui.dialog['confirm'].show()

    def alert(self, msg: any):
        """
        Show alert dialog

        :param msg: message to show
        """
        msg = self.window.core.debug.parse_alert(msg)
        self.window.ui.dialog['alert'].message.setPlainText(msg)
        self.window.ui.dialog['alert'].show()

    def open_editor(self, id: str, data_id: str, width: int = 400, height: int = 400):
        """
        Open editor dialog

        :param id: editor dialog id
        :param data_id: data id
        :param width: dialog width
        :param height: dialog height
        """
        if id not in self.window.ui.dialog:
            return
        self.window.ui.dialog[id].resize(width, height)
        self.window.ui.dialog[id].data_id = data_id
        self.window.ui.dialog[id].show()

    def open_dictionary_editor(
            self,
            id: str,
            option: dict,
            data: dict,
            idx: int,
            width: int = 400,
            height: int = 400
    ):
        """
        Open dictionary editor dialog

        :param id: dictionary editor id (parent id . option key)
        :param option: dictionary option keys
        :param data: data
        :param idx: index on list (needed to save update)
        :param width: dialog width
        :param height: dialog height
        """
        dialog_id = "editor.dictionary." + id
        if dialog_id not in self.window.ui.dialog:
            print("Dialog not found: " + dialog_id)
            return
        self.window.controller.config.dictionary.append_editor(id, option, data)
        self.window.ui.dialog[dialog_id].resize(width, height)
        self.window.ui.dialog[dialog_id].data = data  # store editing data
        self.window.ui.dialog[dialog_id].idx = idx  # store editing record idx
        self.window.ui.dialog[dialog_id].show()

    def register_dictionary(
            self,
            key: str,
            parent: str,
            option: dict
    ):
        """
        Register dictionary data to dictionary editor

        :param key: option key (main dictionary key)
        :param parent: parent id
        :param option: dictionary option keys
        """
        dict_id = parent + "." + key
        self.dictionary.register(dict_id, key, parent, option)

    def open(
            self,
            id: str,
            width: int = 400,
            height: int = 400
    ):
        """
        Open dialog

        :param id: dialog id
        :param width: dialog width
        :param height: dialog height
        """
        if id not in self.window.ui.dialog:
            return
        self.window.ui.dialog[id].resize(width, height)
        qr = self.window.ui.dialog[id].frameGeometry()
        cp = self.window.screen().availableGeometry().center()
        qr.moveCenter(cp)
        self.window.ui.dialog[id].move(qr.topLeft())
        self.window.ui.dialog[id].show()
        self.window.ui.dialog[id].activateWindow()
        self.window.ui.dialog[id].setFocus()

    def open_instance(self,
            id: str,
            width: int = 400,
            height: int = 400,
            type="text_editor"):
        """
        Open edit dialog (multiple instances)

        :param id: dialog id
        :param width: dialog width
        :param height: dialog height
        :param type: dialog type
        """
        if id not in self.window.ui.dialog:
            dialog_instance = self.window.tools.get_instance(type, id)
            if dialog_instance:
                self.window.ui.dialog[id] = dialog_instance
            else:
                print("Dialog not found: " + id)
                return
        self.window.ui.dialog[id].resize(width, height)
        qr = self.window.ui.dialog[id].frameGeometry()
        cp = self.window.screen().availableGeometry().center()
        qr.moveCenter(cp)
        self.window.ui.dialog[id].move(qr.topLeft())
        self.window.ui.dialog[id].show()
        self.window.ui.dialog[id].activateWindow()
        self.window.ui.dialog[id].setFocus()

    def close(self, id: str):
        """
        Close dialog

        :param id: dialog id
        """
        if id not in self.window.ui.dialog:
            return
        self.window.ui.dialog[id].close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.17 20:00:00                  #
# ================================================== #

import os
import webbrowser

from PySide6.QtCore import Qt
from PySide6.QtGui import QPixmap
from PySide6.QtWidgets import QLabel, QVBoxLayout, QPushButton, QPlainTextEdit, QHBoxLayout, QCheckBox

from pygpt_net.ui.widget.element.labels import TitleLabel, CmdLabel
from pygpt_net.utils import trans
from .base import BaseDialog


class UpdateDialog(BaseDialog):
    def __init__(self, window=None, id="update"):
        """
        Update dialog

        :param window: main window
        """
        super(UpdateDialog, self).__init__(window, id)
        self.window = window
        self.setParent(window)
        self.setWindowTitle(trans('update.title'))
        self.cmd_pip = "pip install --upgrade pygpt-net"
        self.cmd_snap = "sudo snap refresh pygpt"

        # www
        self.www = QPushButton(trans('update.download'))
        self.www.setCursor(Qt.PointingHandCursor)
        self.www.clicked.connect(
            lambda: self.window.controller.dialogs.info.goto_update()
        )

        # snap store
        self.snap_store = QPushButton(trans('update.snap'))
        self.snap_store.setCursor(Qt.PointingHandCursor)
        self.snap_store.clicked.connect(
            lambda: self.window.controller.dialogs.info.goto_snap()
        )

        self.changelog = QPlainTextEdit()
        self.changelog.setReadOnly(True)
        self.changelog.setMinimumHeight(200)

        # logo
        logo_label = QLabel()
        path = os.path.abspath(
            os.path.join(self.window.core.config.get_app_path(), 'data', 'logo.png'))
        pixmap = QPixmap(path)
        logo_label.setPixmap(pixmap)

        buttons = QHBoxLayout()
        buttons.addWidget(self.www)
        buttons.addWidget(self.snap_store)

        # checkbox startup
        self.checkbox_startup = QCheckBox(trans("updater.check.launch"))
        self.checkbox_startup.stateChanged.connect(
            lambda: self.window.controller.launcher.toggle_update_check(
                self.checkbox_startup.isChecked())
        )

        if self.window.core.config.get('updater.check.launch'):
            self.checkbox_startup.setChecked(True)
        else:
            self.checkbox_startup.setChecked(False)

        # update cmd/buttons
        self.cmd = CmdLabel(self.window, "")
        self.download_file = QPushButton("Download")
        self.download_file.setCursor(Qt.PointingHandCursor)
        self.download_file.clicked.connect(
            lambda: self.start_download())
        self.download_link = ""

        # info upgrade now
        self.info_upgrade = QLabel(trans("update.info.upgrade"))
        self.info_upgrade.setWordWrap(True)
        self.info_upgrade.setAlignment(Qt.AlignCenter)
        self.info_upgrade.setStyleSheet(
            "font-size: 12px;"
            "margin: 0px 0px 5px 0px;"
        )
        self.info_upgrade.setMaximumHeight(40)

        # layout
        self.layout = QVBoxLayout()
        self.message = QLabel("")
        self.message.setStyleSheet(
            "margin: 10px 0px 10px 0px;"
        )
        self.info = TitleLabel(trans("update.info"))
        self.info.setWordWrap(True)
        self.info.setAlignment(Qt.AlignCenter)
        self.info.setStyleSheet(
            "font-weight: bold;"
            "font-size: 12px;"
            "margin: 10px 0px 10px 0px;"
        )
        self.info.setMaximumHeight(60)
        self.layout.addWidget(logo_label)
        self.layout.addWidget(self.info)
        self.layout.addWidget(self.info_upgrade)
        self.layout.addWidget(self.cmd)
        self.layout.addWidget(self.download_file)
        self.layout.addWidget(self.message)
        self.layout.addWidget(self.changelog, 1)
        self.layout.addWidget(self.checkbox_startup)
        self.layout.addLayout(buttons)
        self.layout.addStretch()
        self.setLayout(self.layout)

    def start_download(self):
        """
        Download file
        """
        webbrowser.open(self.download_link)

    def set_data(
            self,
            is_new: bool,
            version: str,
            build: str,
            changelog: str,
            download_windows: str,
            download_linux: str
    ):
        """
        Set update data

        :param is_new: is new version
        :param version: version
        :param build: build
        :param changelog: changelog
        :param download_windows: download link for windows
        :param download_linux: download link for linux
        """
        # prepare data
        info = trans("update.info")
        if not is_new:
            info = trans('update.info.none')
        txt = trans('update.new_version') + ": " \
            + str(version) + " (" + trans('update.released') + ": " + str(build) + ")"
        txt += "\n" + trans('update.current_version') + ": " + self.window.meta['version']

        self.info.setText(info)
        self.changelog.setPlainText(changelog)
        self.message.setText(txt)

        # show / hide upgrade info
        if is_new:
            self.info_upgrade.setVisible(True)
        else:
            self.info_upgrade.setVisible(False)

        # check platform
        self.cmd.setVisible(False)
        self.download_file.setVisible(False)
        self.snap_store.setVisible(False)

        if is_new:
            if self.window.core.platforms.is_snap():  # snap
                self.cmd.setText(self.cmd_snap)
                self.cmd.setVisible(True)
            elif self.window.core.config.is_compiled():  # compiled
                if self.window.core.platforms.is_windows():
                    self.download_link = download_windows
                    self.download_file.setText("{} .msi ({})".format(trans("action.download"), version))
                    self.download_file.setVisible(True)
                elif self.window.core.platforms.is_linux():
                    self.download_link = download_linux
                    self.download_file.setText("{} .tar.gz ({})".format(trans("action.download"), version))
                    self.download_file.setVisible(True)
            else:  # pip
                self.cmd.setText(self.cmd_pip)
                self.cmd.setVisible(True)

        # show snap store button
        if self.window.core.platforms.is_linux():
            self.snap_store.setVisible(True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.14 21:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction, QIcon, QResizeEvent, QImage
from PySide6.QtWidgets import QMenu, QApplication, QHeaderView

from pygpt_net.ui.widget.lists.base import BaseList
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class AttachmentList(BaseList):
    def __init__(self, window=None, id=None):
        """
        Attachments menu

        :param window: main window
        :param id: input id
        """
        super(AttachmentList, self).__init__(window)
        self.window = window
        self.id = id
        self.doubleClicked.connect(self.dblclick)
        self.setHeaderHidden(False)
        self.clicked.disconnect(self.click)

        self.header = self.header()
        self.header.setStretchLastSection(False)

        self.column_proportion = 0.3
        self.adjustColumnWidths()

    def adjustColumnWidths(self):
        total_width = self.width()
        first_column_width = int(total_width * self.column_proportion)
        self.setColumnWidth(0, first_column_width)
        for column in range(1, 3):
            self.setColumnWidth(column, (total_width - first_column_width) // (3 - 1))

    def resizeEvent(self, event: QResizeEvent):
        super().resizeEvent(event)
        self.adjustColumnWidths()

    def mousePressEvent(self, event):
        """
        Mouse press event

        :param event: mouse event
        """
        if event.buttons() == Qt.LeftButton:
            index = self.indexAt(event.pos())
            if index.isValid():
                mode = self.window.core.config.get('mode')
                self.window.controller.attachment.select(mode, index.row())
        super(AttachmentList, self).mousePressEvent(event)

    def click(self, val):
        """
        Click event

        :param val: click event
        """
        return

    def dblclick(self, val):
        """
        Double click event

        :param val: double click event
        """
        mode = self.window.core.config.get('mode')
        self.window.controller.attachment.select(mode, val.row())

    def contextMenuEvent(self, event):
        """
        Context menu event

        :param event: context menu event
        """
        mode = self.window.core.config.get('mode')
        item = self.indexAt(event.pos())
        idx = item.row()
        preview_actions = []
        path = None

        if idx >= 0:
            path = self.window.controller.attachment.get_path_by_idx(mode, idx)
            preview_actions = []
            if self.window.core.filesystem.actions.has_preview(path):
                preview_actions = self.window.core.filesystem.actions.get_preview(self, path)

        actions = {}
        actions['open'] = QAction(QIcon(":/icons/view.svg"), trans('action.open'), self)
        actions['open'].triggered.connect(
            lambda: self.action_open(event))

        actions['open_dir'] = QAction(QIcon(":/icons/folder_filled.svg"), trans('action.open_dir'), self)
        actions['open_dir'].triggered.connect(
            lambda: self.action_open_dir(event))

        actions['rename'] = QAction(QIcon(":/icons/edit.svg"), trans('action.rename'), self)
        actions['rename'].triggered.connect(
            lambda: self.action_rename(event))

        actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.delete'), self)
        actions['delete'].triggered.connect(
            lambda: self.action_delete(event))

        menu = QMenu(self)
        if idx >= 0 and preview_actions:
            for action in preview_actions:
                menu.addAction(action)
        menu.addAction(actions['open'])
        menu.addAction(actions['open_dir'])

        if idx >= 0:
            if self.window.core.filesystem.actions.has_use(path):
                use_actions = self.window.core.filesystem.actions.get_use(self, path)
                use_menu = QMenu(trans('action.use'), self)
                for action in use_actions:
                    use_menu.addAction(action)
                menu.addMenu(use_menu)

        menu.addAction(actions['rename'])
        menu.addAction(actions['delete'])

        if idx >= 0:
            self.window.controller.attachment.select(mode, item.row())
            menu.exec_(event.globalPos())

    def keyPressEvent(self, event):
        """
        Key press event to handle undo action

        :param event: Event
        """
        if event.key() == Qt.Key_V and QApplication.keyboardModifiers() == Qt.ControlModifier:
            self.handle_paste()

    def handle_paste(self):
        """Handle clipboard paste"""
        clipboard = QApplication.clipboard()
        source = clipboard.mimeData()
        if source.hasImage():
            image = source.imageData()
            if isinstance(image, QImage):
                self.window.controller.attachment.from_clipboard_image(image)
        elif source.hasUrls():
            urls = source.urls()
            for url in urls:
                if url.isLocalFile():
                    local_path = url.toLocalFile()
                    self.window.controller.attachment.from_clipboard_url(local_path, all=True)
        elif source.hasText():
            text = source.text()
            self.window.controller.attachment.from_clipboard_text(text, all=True)

    def action_open(self, event):
        """
        Open action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            mode = self.window.core.config.get('mode')
            self.window.controller.attachment.open(mode, idx)

    def action_open_dir(self, event):
        """
        Open dir action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            mode = self.window.core.config.get('mode')
            self.window.controller.attachment.open_dir(mode, idx)

    def action_rename(self, event):
        """
        Rename action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            mode = self.window.core.config.get('mode')
            self.window.controller.attachment.rename(mode, idx)

    def action_delete(self, event):
        """
        Delete action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.attachment.delete(idx)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.12 06:00:00                  #
# ================================================== #

import datetime


class IndexesDebug:
    def __init__(self, window=None):
        """
        Indexes debug

        :param window: Window instance
        """
        self.window = window
        self.id = 'indexes'

    def update(self):
        """Update debug window."""
        self.window.core.debug.begin(self.id)
        self.window.core.debug.add(self.id, 'Current storage:', str(self.window.core.idx.get_current_store()))
        self.window.core.debug.add(self.id, 'Current idx:', str(self.window.controller.idx.current_idx))
        self.window.core.debug.add(self.id, 'Indexes (list):', str(list(self.window.core.config.get("llama.idx.list"))))

        # count items in DB
        db_counter = {}
        db_counter["ctx"] = self.window.core.idx.get_counters("ctx")
        db_counter["file"] = self.window.core.idx.get_counters("file")
        db_counter["external"] = self.window.core.idx.get_counters("external")

        for store in self.window.core.idx.items:
            name = "Items (files): " + store
            indexes_data = {}
            for idx in self.window.core.idx.items[store]:
                indexes_data[idx] = len(self.window.core.idx.items[store][idx].items)
            if len(indexes_data) > 0:
                self.window.core.debug.add(self.id, name, str(indexes_data))

        # DB items counters
        self.window.core.debug.add(self.id, 'DB items (ctx):',
                                   str(db_counter["ctx"]))
        self.window.core.debug.add(self.id, 'DB items (external/web):',
                                   str(db_counter["external"]))
        self.window.core.debug.add(self.id, 'DB items (file):',
                                   str(db_counter["file"]))

        self.window.core.debug.add(self.id, 'Storage (storages):',
                                   str(list(self.window.core.idx.storage.storages.keys())))
        self.window.core.debug.add(self.id, 'Temp (in-memory) indices:',
                                   str(self.window.core.idx.storage.count_tmp()))

        # loaders
        self.window.core.debug.add(self.id, 'Offline loaders [files]:',
                                   str(sorted(list(self.window.core.idx.indexing.loaders["file"].keys()))))
        self.window.core.debug.add(self.id, 'Offline loaders [web]:',
                                   str(sorted(list(self.window.core.idx.indexing.loaders["web"].keys()))))
        self.window.core.debug.add(self.id, 'External instructions [web]:',
                                   str(self.window.core.idx.indexing.external_instructions))

        excluded = self.window.core.config.get("llama.idx.excluded.ext").replace(" ", "").split(',')
        self.window.core.debug.add(self.id, 'Excluded (ext):', str(excluded))
        self.window.core.debug.add(self.id, 'Force exclude:',
                                   str(self.window.core.config.get("llama.idx.excluded.force")))
        self.window.core.debug.add(self.id, 'Custom metadata:',
                                   str(self.window.core.config.get("llama.idx.custom_meta")))

        # ctx
        self.window.core.debug.add(self.id, 'CTX [auto]:',
                                   str(self.window.core.config.get("llama.idx.auto")))
        last_str = "-"
        last_ctx = int(self.window.core.config.get("llama.idx.db.last"))
        if last_ctx > 0:
            last_str = datetime.datetime.fromtimestamp(last_ctx).strftime('%Y-%m-%d %H:%M:%S') + " (" + str(last_ctx) + ")"
        self.window.core.debug.add(self.id, 'CTX [db.last]:', str(last_str))

        # indexes
        """
        indexes = self.window.core.idx.get_all()
        for key in list(indexes):
            path = os.path.join(self.window.core.config.get_user_dir('idx'), key)
            idx = indexes[key]
            self.window.core.debug.add(self.id, '----', '')
            idx_data = {
                'id': idx.id,
                'store': idx.store,
                'path': path,
                'items': len(idx.items),
            }
            self.window.core.debug.add(self.id, 'IDX: [' + str(key) + ']', str(idx_data))

            # files
            items = idx.items
            for item_id in items:
                item = items[item_id]
                indexed_dt = datetime.datetime.fromtimestamp(item['indexed_ts'])
                data = {
                    'id': item['id'],
                    'path': item['path'],
                    'indexed_at': str(item['indexed_ts']) + ' (' + str(indexed_dt) + ')',
                    'file_id': item_id,
                }
                self.window.core.debug.add(self.id, item_id, str(data))
        """

        self.window.core.debug.end(self.id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.14 21:00:00                  #
# ================================================== #

import os

from PySide6 import QtCore
from PySide6.QtGui import QStandardItemModel, Qt
from PySide6.QtWidgets import QVBoxLayout, QPushButton, QHBoxLayout, QCheckBox, QLabel, QWidget

from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.ui.widget.lists.uploaded import UploadedFileList
from pygpt_net.utils import trans


class AttachmentsUploaded:
    def __init__(self, window=None):
        """
        Attachments Uploaded UI

        :param window: Window instance
        """
        self.window = window
        self.id = 'attachments_uploaded'

    def setup(self) -> QVBoxLayout:
        """
        Setup list

        :return: QVBoxLayout
        """
        self.setup_attachments()

        self.window.ui.nodes['attachments_uploaded.sync.tip'] = QLabel(trans('attachments_uploaded.sync.tip'))
        self.window.ui.nodes['attachments_uploaded.sync.tip'].setAlignment(Qt.AlignCenter)
        empty_widget = QWidget()

        self.window.ui.nodes['tip.input.attachments.uploaded'] = HelpLabel(trans('tip.input.attachments.uploaded'),
                                                                           self.window)

        # buttons layout
        buttons_layout = QHBoxLayout()
        buttons_layout.addWidget(self.window.ui.nodes['attachments_uploaded.btn.sync'])
        buttons_layout.addWidget(self.window.ui.nodes['attachments_uploaded.btn.clear'])
        buttons_layout.addWidget(empty_widget)
        buttons_layout.addWidget(self.window.ui.nodes['attachments_uploaded.sync.tip'])

        # layout
        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes['tip.input.attachments.uploaded'])
        layout.addWidget(self.window.ui.nodes['attachments_uploaded'])
        layout.addLayout(buttons_layout)

        return layout

    def setup_attachments(self):
        """
        Setup attachments uploaded list
        """
        # attachments
        self.window.ui.nodes[self.id] = UploadedFileList(self.window)

        # buttons
        self.window.ui.nodes['attachments_uploaded.btn.sync'] = QPushButton(trans('attachments_uploaded.btn.sync'))
        self.window.ui.nodes['attachments_uploaded.btn.clear'] = QPushButton(trans('attachments_uploaded.btn.clear'))

        self.window.ui.nodes['attachments_uploaded.btn.sync'].clicked.connect(
            lambda: self.window.controller.assistant.files.sync())
        self.window.ui.nodes['attachments_uploaded.btn.clear'].clicked.connect(
            lambda: self.window.controller.assistant.files.clear_files())

        self.window.ui.models[self.id] = self.create_model(self.window)
        self.window.ui.nodes[self.id].setModel(self.window.ui.models[self.id])

    def create_model(self, parent) -> QStandardItemModel:
        """
        Create list model

        :param parent: parent widget
        :return: QStandardItemModel
        """
        model = QStandardItemModel(0, 3, parent)
        model.setHeaderData(0, Qt.Horizontal, trans('attachments.header.name'))
        model.setHeaderData(1, Qt.Horizontal, trans('attachments.header.path'))
        model.setHeaderData(2, Qt.Horizontal, trans('attachments.header.size'))
        return model

    def update(self, data):
        """
        Update list

        :param data: Data to update
        """
        self.window.ui.models[self.id].removeRows(0, self.window.ui.models[self.id].rowCount())
        i = 0
        for id in data:
            size = "-"
            if 'name' not in data[id] or 'path' not in data[id]:
                continue
            path = data[id]['path']
            if 'size' in data[id] and data[id]['size'] is not None:
                size = self.window.core.filesystem.sizeof_fmt(data[id]['size'])
            else:
                if path and os.path.exists(path):
                    size = self.window.core.filesystem.sizeof_fmt(os.path.getsize(path))
            self.window.ui.models[self.id].insertRow(i)
            index = self.window.ui.models[self.id].index(i, 0)
            self.window.ui.models[self.id].setData(index, "ID: " + id, QtCore.Qt.ToolTipRole)
            self.window.ui.models[self.id].setData(self.window.ui.models[self.id].index(i, 0), data[id]['name'])
            self.window.ui.models[self.id].setData(self.window.ui.models[self.id].index(i, 1), path)
            self.window.ui.models[self.id].setData(self.window.ui.models[self.id].index(i, 2), size)
            i += 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.10 23:00:00                  #
# ================================================== #

from PySide6 import QtCore
from PySide6.QtWidgets import QLineEdit


class FindInput(QLineEdit):
    def __init__(self, window=None, id=None):
        """
        Find input

        :param window: main window
        :param id: info window id
        """
        super(FindInput, self).__init__(window)

        self.window = window
        self.id = id
        self.textChanged.connect(
            lambda: self.window.controller.finder.search_text_changed(self.text()),
        )

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(FindInput, self).keyPressEvent(event)

        # update on Enter
        if event.key() == QtCore.Qt.Key_Return or event.key() == QtCore.Qt.Key_Enter:
            self.window.controller.finder.focus_input(self.text())

    def focusInEvent(self, e):
        """
        Focus in event

        :param e: focus event
        """
        super(FindInput, self).focusInEvent(e)
        self.window.controller.finder.focus_input(self.text())




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.15 12:00:00                  #
# ================================================== #

from pygpt_net.utils import trans


class Plugins:
    def __init__(self, window=None):
        """
        Plugins locale controller

        :param window: Window instance
        """
        self.window = window

    def apply(self):
        """Apply locale to plugins"""

        # plugins: info
        self.window.controller.plugins.update_info()

        # reload all domains (plugin locale files)
        ids = self.window.core.plugins.plugins.keys()
        for id in ids:
            plugin = self.window.core.plugins.plugins[id]
            if not plugin.use_locale:
                continue
            domain = 'plugin.{}'.format(id)
            trans('', True, domain)

        # apply to plugin settings
        for id in ids:
            plugin = self.window.core.plugins.plugins[id]
            if not plugin.use_locale:
                continue
            domain = 'plugin.{}'.format(id)

            # set name, translate if localization is enabled
            name_txt = trans('plugin.name', False, domain)

            # set description, translate if localization is enabled
            desc_key = 'plugin.settings.' + id + '.desc'
            desc_txt = trans('plugin.description', False, domain)
            if desc_key in self.window.ui.nodes:
                self.window.ui.nodes[desc_key].setText(desc_txt)

            # update tab name
            tab_idx = self.window.controller.plugins.get_tab_idx(id)
            # update tab name
            if tab_idx is not None:
                self.window.ui.tabs['plugin.settings'].setTabText(tab_idx, name_txt)

            if id in self.window.ui.menu['plugins']:
                self.window.ui.menu['plugins'][id].setText(name_txt)

            options = plugin.setup()
            option_ids = options.keys()
            for option_id in option_ids:
                # prepare element nodes keys
                label_key = 'plugin.' + id + '.' + option_id + '.label'
                desc_key = 'plugin.' + id + '.' + option_id + '.desc'

                # update options label, description and tooltip
                label_str = trans(option_id + '.label', False, domain)
                desc_str = trans(option_id + '.description', False, domain)
                tooltip_str = trans(option_id + '.tooltip', False, domain)

                if tooltip_str == option_id + '.tooltip':
                    tooltip_str = desc_str
                if label_key in self.window.ui.nodes:
                    self.window.ui.nodes[label_key].setText(label_str)
                if desc_key in self.window.ui.nodes:
                    self.window.ui.nodes[desc_key].setText(desc_str)
                    self.window.ui.nodes[desc_key].setToolTip(tooltip_str)

                if options[option_id]['type'] == 'bool':
                    # update checkbox label
                    if domain in self.window.ui.config and option_id in self.window.ui.config[domain]:
                        try:
                            self.window.ui.config[domain][option_id].box.setText(label_str)
                        except Exception as e:
                            pass

        # update settings dialog list
        idx = self.window.ui.tabs['plugin.settings'].currentIndex()
        self.window.plugin_settings.update_list('plugin.list', self.window.core.plugins.plugins)
        self.window.controller.plugins.set_by_tab(idx)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.20 06:00:00                  #
# ================================================== #

from PySide6.QtCore import QUrl
from PySide6.QtGui import QDesktopServices

class Url:
    def __init__(self, window=None):
        """
        Filesystem URL handler

        :param window: Window instance
        """
        self.window = window

    def handle(self,  url: QUrl):
        """
        Handle URL

        :param url: url
        """
        extra_schemes = [
            'extra-audio-read',
            'extra-code-copy',
            'extra-copy',
            'extra-delete',
            'extra-edit',
            'extra-join',
            'extra-replay',
        ]

        # JS bridge
        if url.toString() == 'bridge://open_find':
            self.window.ui.nodes['output'].find_open()
            return

        # -------------

        # local file
        if not url.scheme().startswith('http') and url.scheme() not in extra_schemes:
            self.window.controller.files.open(url.toLocalFile())

        # extra actions
        elif url.scheme() == 'extra-delete':  # ctx item delete
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.delete_item(int(id))
        elif url.scheme() == 'extra-edit':  # ctx item edit
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.edit_item(int(id))
        elif url.scheme() == 'extra-copy':  # ctx item copy
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.copy_item(int(id))
        elif url.scheme() == 'extra-replay':  # ctx regen response
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.replay_item(int(id))
        elif url.scheme() == 'extra-audio-read':  # ctx audio read
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.audio_read_item(int(id))
        elif url.scheme() == 'extra-join':  # ctx join
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.join_item(int(id))
        elif url.scheme() == 'extra-code-copy':  # copy code block
            id = url.toString().split(':')[1]
            self.window.controller.ctx.extra.copy_code_block(int(id))

        else:
            # external link
            QDesktopServices.openUrl(url)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.25 01:00:00                  #
# ================================================== #

import os
import requests

from .base import BaseProvider


class ElevenLabsTextToSpeech(BaseProvider):
    def __init__(self, *args, **kwargs):
        """
        Eleven Labs Text to Speech provider

        :param args: args
        :param kwargs: kwargs
        """
        super(ElevenLabsTextToSpeech, self).__init__(*args, **kwargs)
        self.plugin = kwargs.get("plugin")
        self.id = "eleven_labs"
        self.name = "Eleven Labs"

    def init_options(self):
        """Initialize options"""
        url_api = {
            "API Key": "https://elevenlabs.io/speech-synthesis",
        }
        self.plugin.add_option(
            "eleven_labs_api_key",
            type="text",
            value="",
            label="Eleven Labs API Key",
            tab="eleven_labs",
            description="You can obtain your own API key at: "
                        "https://elevenlabs.io/speech-synthesis",
            tooltip="Eleven Labs API Key",
            secret=True,
            persist=True,
            urls=url_api,
        )
        self.plugin.add_option(
            "eleven_labs_voice",
            type="text",
            value="kigFud3tYib8QFaSSi9D",
            label="Voice ID",
            tab="eleven_labs",
            description="Specify Voice ID",
            tooltip="Voice ID",
            urls={
                "Voices": "https://elevenlabs.io/voice-library"
            },
        )
        self.plugin.add_option(
            "eleven_labs_model",
            type="text",
            value="eleven_multilingual_v2",
            label="Model",
            tab="eleven_labs",
            description="Specify model",
            tooltip="Model name",
            urls={
                "Models": "https://elevenlabs.io/docs/speech-synthesis/models"
            },
        )

    def speech(self, text: str) -> str:
        """
        Speech text to audio

        :param text: text to speech
        :return: path to generated audio file or None if audio playback is handled here
        """
        api_key = self.plugin.get_option_value("eleven_labs_api_key")
        model = self.plugin.get_option_value("eleven_labs_model")
        voice = self.plugin.get_option_value("eleven_labs_voice")
        output_file = self.plugin.output_file
        path = os.path.join(
            self.plugin.window.core.config.path,
            output_file,
        )
        CHUNK_SIZE = 1024
        url = "https://api.elevenlabs.io/v1/text-to-speech/" + voice

        headers = {
            "Accept": "audio/mpeg",
            "Content-Type": "application/json",
            "xi-api-key": api_key,
        }
        data = {
            "text": text,
            "model_id": model,
            "voice_settings": {
                "stability": 0.5,
                "similarity_boost": 0.5,
            }
        }
        response = requests.post(url, json=data, headers=headers)
        with open(path, 'wb') as f:
            for chunk in response.iter_content(chunk_size=CHUNK_SIZE):
                if chunk:
                    f.write(chunk)
        return path

    def is_configured(self) -> bool:
        """
        Check if provider is configured

        :return: True if configured, False otherwise
        """
        api_key = self.plugin.get_option_value("eleven_labs_api_key")
        return api_key is not None and api_key != ""

    def get_config_message(self) -> str:
        """
        Return message to display when provider is not configured

        :return: message
        """
        api_key = self.plugin.get_option_value("eleven_labs_api_key")
        if api_key is None or api_key == "":
            return "Eleven Labs API KEY is not set. Please set it in plugin settings."




# !/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.23 19:00:00                  #
# ================================================== #

class Vision:
    def __init__(self, window=None):
        """
        UI vision update controller

        :param window: Window instance
        """
        self.window = window

    def update(self):
        """Update vision options"""
        mode = self.window.core.config.data['mode']
        if self.window.controller.painter.is_active():
            self.window.controller.camera.setup()
            self.window.controller.camera.show_camera()
        else:
            # plugin: vision
            if self.window.controller.plugins.is_type_enabled('vision'):
                if mode == 'vision' or mode in self.window.controller.chat.vision.allowed_modes:
                    self.window.controller.camera.setup()
                    self.window.controller.camera.show_camera()
                    if mode != 'vision':
                        self.window.controller.chat.vision.show_inline()
                else:
                    self.window.controller.camera.hide_camera()
                    self.window.controller.chat.vision.hide_inline()
            # no-plugin
            else:
                if mode != 'vision':
                    self.window.controller.camera.hide_camera()
                    self.window.controller.chat.vision.hide_inline()
                else:
                    self.window.controller.camera.setup()
                    self.window.controller.camera.show_camera()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.06 02:00:00                  #
# ================================================== #

from pygpt_net.utils import unpack_var


def unpack_file_item(row: dict) -> (str, dict):
    """
    Unpack item from DB row

    :param row: DB row
    :return: idx, file data
    """
    data = {}
    idx = row['idx']
    data["db_id"] = unpack_var(row['id'], 'int')
    data["id"] = row['doc_id']
    data["name"] = row['name']
    data["path"] = row['path']
    data["indexed_ts"] = unpack_var(row['updated_ts'], 'int')
    return idx, data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.03 22:00:00                  #
# ================================================== #

from sqlalchemy import text

from .base import BaseMigration


class Version20240303190000(BaseMigration):
    def __init__(self, window=None):
        super(Version20240303190000, self).__init__(window)
        self.window = window

    def up(self, conn):
        conn.execute(text("""
        ALTER TABLE ctx_item ADD COLUMN docs_json TEXT;
        """))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.16 16:00:00                  #
# ================================================== #

import copy
import os
import shutil
import json
import ssl
import time

from urllib.request import urlopen, Request

from PySide6.QtCore import QObject, Signal, Slot, QRunnable, QTimer
from packaging.version import parse as parse_version, Version

from pygpt_net.utils import trans


class Updater:
    def __init__(self, window=None):
        """
        Updater core (config data patcher)

        :param window: Window instance
        """
        self.window = window

    def patch(self):
        """Patch config data to current version"""
        try:
            version = self.get_app_version()

            # migrate DB
            self.migrate_db()

            self.patch_config(version)
            self.patch_models(version)
            self.patch_presets(version)
            self.patch_ctx(version)
            self.patch_indexes(version)
            self.patch_assistants(version)
            self.patch_attachments(version)
            self.patch_notepad(version)
        except Exception as e:
            self.window.core.debug.log(e)
            print("Failed to patch config data!")

    def migrate_db(self):
        """Migrate database"""
        try:
            self.window.core.db.migrate()
        except Exception as e:
            self.window.core.debug.log(e)
            print("Failed to migrate database!")

    def patch_config(self, version: Version):
        """
        Migrate config to current app version

        :param version: current app version
        """
        if self.window.core.config.patch(version):
            print("Migrated config. [OK]")

    def patch_models(self, version: Version):
        """
        Migrate models to current app version

        :param version: current app version
        """
        if self.window.core.models.patch(version):
            print("Migrated models. [OK]")

    def patch_presets(self, version: Version):
        """
        Migrate presets to current app version

        :param version: current app version
        """
        if self.window.core.presets.patch(version):
            print("Migrated presets. [OK]")

    def patch_ctx(self, version: Version):
        """
        Migrate ctx to current app version

        :param version: current app version
        """
        if self.window.core.ctx.patch(version):
            print("Migrated ctx. [OK]")

    def patch_assistants(self, version: Version):
        """
        Migrate assistants to current app version

        :param version: current app version
        """
        if self.window.core.assistants.patch(version):
            print("Migrated assistants. [OK]")

    def patch_attachments(self, version: Version):
        """
        Migrate attachments to current app version

        :param version: current app version
        """
        if self.window.core.attachments.patch(version):
            print("Migrated attachments. [OK]")

    def patch_indexes(self, version: Version):
        """
        Migrate indexes to current app version

        :param version: current app version
        """
        if self.window.core.idx.patch(version):
            print("Migrated indexes. [OK]")

    def patch_notepad(self, version: Version):
        """
        Migrate notepad to current app version

        :param version: current app version
        """
        if self.window.core.notepad.patch(version):
            print("Migrated notepad. [OK]")

    def patch_dir(self, dir_name: str = "", force: bool = False):
        """
        Patch directory (replace all files)

        :param dir_name: directory name
        :param force: force update
        """
        try:
            # directory
            dst_dir = os.path.join(self.window.core.config.path, dir_name)
            src = os.path.join(self.window.core.config.get_app_path(), 'data', 'config', dir_name)
            for file in os.listdir(src):
                src_file = os.path.join(src, file)
                dst_file = os.path.join(dst_dir, file)
                if not os.path.exists(dst_file) or force:
                    shutil.copyfile(src_file, dst_file)
                    print("Patched file: {}.".format(dst_file))
        except Exception as e:
            self.window.core.debug.log(e)

    def patch_file(self, filename: str = "", force: bool = False):
        """
        Patch file

        :param filename: file name
        :param force: force update
        """
        try:
            # file
            dst = os.path.join(self.window.core.config.path, filename)
            if not os.path.exists(dst) or force:
                src = os.path.join(self.window.core.config.get_app_path(), 'data', 'config', filename)
                # make backup of old file
                if os.path.exists(dst):
                    shutil.copyfile(dst, dst + '.backup')
                    print("Backup file: {}.".format(dst + '.backup'))
                shutil.copyfile(src, dst)
                print("Patched file: {}.".format(dst))
        except Exception as e:
            self.window.core.debug.log(e)

    def patch_css(self, filename: str = "", force: bool = False):
        """
        Patch css file

        :param filename: file name
        :param force: force update
        """
        try:
            # file
            dst = os.path.join(self.window.core.config.path, 'css', filename)
            if not os.path.exists(dst) or force:
                src = os.path.join(self.window.core.config.get_app_path(), 'data', 'css', filename)
                # make backup of old file
                if os.path.exists(dst):
                    shutil.copyfile(dst, dst + '.backup')
                    print("Backup css file: {}.".format(dst + '.backup'))
                shutil.copyfile(src, dst)
                print("Patched css file: {}.".format(dst))
        except Exception as e:
            self.window.core.debug.log(e)

    def get_app_version(self) -> Version:
        """
        Get the current app version.

        :return: version (packaging.version.Version)
        """
        return parse_version(self.window.meta['version'])

    def get_updater_url(self) -> str:
        """
        Get the app updates info url.

        :return: updater url
        """
        return self.window.meta['website'] + "/api/version?v=" + str(self.window.meta['version'])

    def check_silent(self) -> (bool, str, str, str, str, str):
        """
        Check version in background

        :return: (is_new, newest_version, newest_build, changelog, download_windows, download_linux)
        """
        url = self.get_updater_url()
        is_new = False
        newest_version = ""
        newest_build = ""
        changelog = ""
        download_windows = ""
        download_linux = ""

        try:
            ctx = ssl.create_default_context()
            ctx.check_hostname = True
            req = Request(
                url=url,
                headers={'User-Agent': 'Mozilla/5.0'},
            )
            response = urlopen(req, context=ctx, timeout=5)
            data_json = json.loads(response.read())
            newest_version = data_json["version"]
            newest_build = data_json["build"]

            # changelog, download links
            changelog = ""
            download_windows = ""
            download_linux = ""
            if "changelog" in data_json:
                changelog = data_json["changelog"]
            if "download_windows" in data_json:
                download_windows = data_json["download_windows"]
            if "download_linux" in data_json:
                download_linux = data_json["download_linux"]

            parsed_newest_version = parse_version(newest_version)
            parsed_current_version = parse_version(self.window.meta['version'])
            if parsed_newest_version > parsed_current_version:
                is_new = parsed_newest_version > parsed_current_version

            # save last check time and version
            self.window.core.config.set("updater.check.bg.last_time", time.time())
            self.window.core.config.set("updater.check.bg.last_version", newest_version)

        except Exception as e:
            self.window.core.debug.log(e)
            print("Failed to check for updates")

        return is_new, newest_version, newest_build, changelog, download_windows, download_linux

    def check(self, force: bool = False) -> bool:
        """
        Check for updates

        :param force: force show version dialog
        :return: True if force show version dialog
        """
        print("Checking for updates...")
        is_new, version, build, changelog, download_windows, download_linux = self.check_silent()
        if is_new or force:
            self.show_version_dialog(
                version,
                build,
                changelog,
                download_windows,
                download_linux,
                is_new
            )
            return True
        print("No updates available.")
        return False

    def show_version_dialog(
            self,
            version: str,
            build: str,
            changelog: str,
            download_windows: str,
            download_linux: str,
            is_new: bool = False
    ):
        """
        Display version dialog

        :param version: version number
        :param build: build date
        :param changelog: changelog
        :param download_windows: windows download link
        :param download_linux: linux download link
        :param is_new: True if is new version available
        """
        self.window.ui.dialog['update'].set_data(
            is_new,
            version,
            build,
            changelog,
            download_windows,
            download_linux
        )
        self.window.ui.dialogs.open('update', height=600)

    def post_check_config(self) -> bool:
        """
        Check for missing config keys and add them.

        :return: True if updated
        """
        base = self.window.core.config.get_base()
        data = self.window.core.config.all()
        updated = False

        # check for any missing keys
        for key in base:
            if key not in data:
                data[key] = copy.deepcopy(base[key])  # copy base value
                updated = True

        # update file
        if updated:
            data = dict(sorted(data.items()))
            self.window.core.config.data = data
            self.window.core.config.save()

        return updated

    @Slot(str, str, str, str, str)
    def handle_new_version(
            self,
            version: str,
            build: str,
            changelog: str,
            download_windows: str = "",
            download_linux: str = ""
    ):
        """
        Handle new version signal

        :param version: version number
        :param build: build date
        :param changelog: changelog
        :param download_windows: download link for windows
        :param download_linux: download link for linux
        """
        if self.window.ui.tray.is_tray:
            self.window.ui.tray.show_msg(
                trans("notify.update.title"),
                version + " (" + build + ")",
            )

        # always show dialog
        self.show_version_dialog(
            version,
            build,
            changelog,
            download_windows,
            download_linux,
            True
        )

    def run_check(self):
        """Run check for updates in background"""
        worker = UpdaterWorker()
        worker.window = self.window
        worker.checker = self.check_silent
        worker.signals.version_changed.connect(self.handle_new_version)
        self.window.threadpool.start(worker)


class UpdaterSignals(QObject):
    version_changed = Signal(str, str, str, str, str)


class UpdaterWorker(QRunnable):
    def __init__(self, *args, **kwargs):
        super(UpdaterWorker, self).__init__()
        self.signals = UpdaterSignals()
        self.args = args
        self.kwargs = kwargs
        self.last_check_time = None
        self.window = None
        self.checker = None

    @Slot()
    def run(self):
        # if background check is not enabled, abort
        if not self.window.core.config.get("updater.check.bg"):
            return
        try:
            # check
            parsed_prev_checked = None
            last_checked = self.window.core.config.get("updater.check.bg.last_version")
            if last_checked is not None and last_checked != "":
                parsed_prev_checked = parse_version(last_checked)

            # print("Checking for updates...")
            is_new, version, build, changelog, download_windows, download_linux = self.checker()
            if is_new:
                if parsed_prev_checked is None or parsed_prev_checked < parse_version(version):
                    self.signals.version_changed.emit(
                        version,
                        build,
                        changelog,
                        download_windows,
                        download_linux
                    )
        except Exception as e:
            self.window.core.debug.log(e)
            print("Failed to check for updates")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.16 12:00:00                  #
# ================================================== #

from pygpt_net.plugin.base import BasePlugin
from pygpt_net.core.dispatcher import Event
from pygpt_net.item.ctx import CtxItem

from .worker import Worker


class Plugin(BasePlugin):
    def __init__(self, *args, **kwargs):
        super(Plugin, self).__init__(*args, **kwargs)
        self.id = "cmd_files"
        self.name = "Command: Files I/O"
        self.description = "Provides commands to read and write files"
        self.order = 100
        self.allowed_cmds = [
            "read_file",
            "query_file",
            "save_file",
            "append_file",
            "delete_file",
            "list_dir",
            "mkdir",
            "download_file",
            "rmdir",
            "copy_file",
            "copy_dir",
            "move",
            "find",
            "is_dir",
            "is_file",
            "file_exists",
            "file_size",
            "file_info",
            "send_file",
            "cwd",
            "file_index",
        ]
        self.use_locale = True
        self.init_options()

    def init_options(self):
        """Initialize options"""
        self.add_option(
            "model_tmp_query",
            type="combo",
            value="gpt-3.5-turbo",
            label="Model for query in-memory index",
            description="Model used for query in-memory index for `query_file` command, "
                        "default: gpt-3.5-turbo",
            tooltip="Query model",
            use="models",
            tab="indexing",
        )
        self.add_option(
            "idx",
            type="text",
            value="base",
            label="Index to use when indexing files",
            description="ID of index to use for files indexing",
            tooltip="Index name",
            tab="indexing",
        )
        self.add_option(
            "use_loaders",
            type="bool",
            value=True,
            label="Use data loaders",
            description="Use data loaders from Llama-index for file reading (read_file command)",
        )
        self.add_option(
            "auto_index",
            type="bool",
            value=False,
            label="Auto index reading files",
            description="If enabled, every time file is read, it will be automatically indexed",
            tab="indexing",
        )
        self.add_option(
            "only_index",
            type="bool",
            value=False,
            label="Only index reading files",
            description="If enabled, file will be indexed without reading it",
            tab="indexing",
        )

        # commands
        self.add_cmd(
            "send_file",
            instruction="send file as attachment from my computer to you for analyze",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Upload file as attachment",
        )
        self.add_cmd(
            "read_file",
            instruction="read data from files",
            params=[
                {
                    "name": "path",
                    "type": "list",
                    "description": "path(s) to files",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Read file",
        )
        self.add_cmd(
            "query_file",
            instruction="read, index and quick query file for additional context",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
                {
                    "name": "query",
                    "type": "str",
                    "description": "query",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Query file with Llama-index",
            tab="indexing",
        )
        self.add_cmd(
            "save_file",
            instruction="save data to file",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path to file",
                    "required": True,
                },
                {
                    "name": "data",
                    "type": "str",
                    "description": "text data",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Save file",
        )
        self.add_cmd(
            "append_file",
            instruction="append data to file",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
                {
                    "name": "data",
                    "type": "str",
                    "description": "data",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Append to file",
        )
        self.add_cmd(
            "delete_file",
            instruction="delete file",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Delete file",
        )
        self.add_cmd(
            "list_dir",
            instruction="list files and dirs",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: List files in directory (ls)",
        )
        self.add_cmd(
            "mkdir",
            instruction="create directory",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Directory creation (mkdir)",
        )
        self.add_cmd(
            "download_file",
            instruction="download file",
            params=[
                {
                    "name": "src",
                    "type": "str",
                    "description": "source URL",
                    "required": True,
                },
                {
                    "name": "dst",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Downloading files",
        )
        self.add_cmd(
            "rmdir",
            instruction="remove directory",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Removing directories",
        )
        self.add_cmd(
            "copy_file",
            instruction="copy file",
            params=[
                {
                    "name": "src",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
                {
                    "name": "dst",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Copying files",
        )
        self.add_cmd(
            "copy_dir",
            instruction="recursive copy directory",
            params=[
                {
                    "name": "src",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
                {
                    "name": "dst",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Copying directories (recursive)",
        )
        self.add_cmd(
            "move",
            instruction="move file or directory",
            params=[
                {
                    "name": "src",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
                {
                    "name": "dst",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Move files and directories (rename)",
        )
        self.add_cmd(
            "is_dir",
            instruction="check if path is directory",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Check if path is directory",
        )
        self.add_cmd(
            "is_file",
            instruction="check if path is file",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Check if path is file",
        )
        self.add_cmd(
            "file_exists",
            instruction="check if file or directory exists",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Check if file or directory exists",
        )
        self.add_cmd(
            "file_size",
            instruction="get file size",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Get file size",
        )
        self.add_cmd(
            "file_info",
            instruction="get file info",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Get file info",
        )
        self.add_cmd(
            "cwd",
            instruction="get current working directory (abs path)",
            params=[],
            enabled=True,
            description="Enable: Get current working directory (cwd)",
        )
        self.add_cmd(
            "file_index",
            instruction="index (embed as vectors in vector DB) file or directory",
            params=[
                {
                    "name": "path",
                    "type": "str",
                    "description": "path",
                    "required": True,
                },
            ],
            enabled=True,
            description="If enabled, model will be able to index file or directory using Llama-index",
        )
        self.add_cmd(
            "find",
            instruction="find file or directory, use empty path to search in current dir",
            params=[
                {
                    "name": "pattern",
                    "type": "str",
                    "description": "name pattern",
                    "required": True,
                },
                {
                    "name": "path",
                    "type": "str",
                    "description": "search directory",
                    "required": True,
                },
                {
                    "name": "recursive",
                    "type": "bool",
                    "description": "recursive search",
                    "required": True,
                },
            ],
            enabled=True,
            description="Enable: Find file or directory",
        )

    def setup(self) -> dict:
        """
        Return available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attach window

        :param window: Window instance
        """
        self.window = window

    def handle(self, event: Event, *args, **kwargs):
        """
        Handle dispatched event

        :param event: event object
        :param args: event args
        :param kwargs: event kwargs
        """
        name = event.name
        data = event.data
        ctx = event.ctx

        if name == Event.CMD_SYNTAX:
            self.cmd_syntax(data)

        elif name == Event.CMD_EXECUTE:
            self.cmd(
                ctx,
                data['commands'],
            )

    def cmd_syntax(self, data: dict):
        """
        Event: CMD_SYNTAX

        :param data: event data dict
        """
        for option in self.allowed_cmds:
            if self.has_cmd(option):
                data['cmd'].append(self.get_cmd(option))  # append command

    def cmd(self, ctx: CtxItem, cmds: list):
        """
        Event: CMD_EXECUTE

        :param ctx: CtxItem
        :param cmds: commands dict
        """
        is_cmd = False
        my_commands = []
        for item in cmds:
            if item["cmd"] in self.allowed_cmds:
                my_commands.append(item)
                is_cmd = True

        if not is_cmd:
            return

        try:
            # worker
            worker = Worker()
            worker.plugin = self
            worker.cmds = my_commands
            worker.ctx = ctx

            # signals (base handlers)
            worker.signals.finished.connect(self.handle_finished)
            worker.signals.log.connect(self.handle_log)
            worker.signals.debug.connect(self.handle_debug)
            worker.signals.status.connect(self.handle_status)
            worker.signals.error.connect(self.handle_error)

            # check if async allowed
            if not self.window.core.dispatcher.async_allowed(ctx):
                worker.run()
                return

            # start
            self.window.threadpool.start(worker)

        except Exception as e:
            self.error(e)

    def read_as_text(self, path: str, use_loaders: bool = True) -> str:
        """
        Read file and return content as text

        :param path: file path
        :param use_loaders: use Llama-index loader to read file
        :return: text content
        """
        if use_loaders:
            return str(self.window.core.idx.indexing.read_text_content(path))
        else:
            with open(path, 'r', encoding="utf-8") as file:
                data = file.read()
            return data

    def log(self, msg: str):
        """
        Log message to console

        :param msg: message to log
        """
        full_msg = '[CMD] ' + str(msg)
        self.debug(full_msg)
        self.window.ui.status(full_msg)
        if self.is_log():
            print(full_msg)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.20 06:00:00                  #
# ================================================== #

class Slider:
    def __init__(self, window=None):
        """
        Slider field handler

        :param window: Window instance
        """
        self.window = window

    def apply(
            self,
            parent_id: str,
            key: str,
            option: dict,
            type: str = None
    ):
        """
        Apply value to slider

        :param parent_id: Options parent ID
        :param key: Option key
        :param option: Option data
        :param type: Update from type (slider, input, None=value)
        """
        value = option["value"]
        is_integer = False
        multiplier = 1

        if "type" in option and option["type"] == "int":
            is_integer = True
        if "multiplier" in option:
            multiplier = option["multiplier"]

        if type != "slider":
            if is_integer:
                try:
                    value = round(int(value), 0)
                except Exception as e:
                    value = 0
            else:
                try:
                    value = float(value)
                except Exception as e:
                    value = 0.0

            if "min" in option and value < option["min"]:
                value = option["min"]
            elif "max" in option and value > option["max"]:
                value = option["max"]

            # update connected input field
            self.window.ui.config[parent_id][key].input.setText(str(value))

        slider_value = round(float(value) * multiplier, 0)

        # from slider
        if type == "slider":
            input_value = value / multiplier
            if is_integer:
                input_value = round(int(input_value), 0)
            txt = "{}".format(input_value)
            self.window.ui.config[parent_id][key].input.setText(txt)

        # from input
        elif type == "input":
            if "min" in option and slider_value < option["min"] * multiplier:
                slider_value = option["min"] * multiplier
            elif "max" in option and slider_value > option["max"] * multiplier:
                slider_value = option["max"] * multiplier
            self.window.ui.config[parent_id][key].slider.setValue(slider_value)

        # from value
        else:
            self.window.ui.config[parent_id][key].input.setText(str(value))
            self.window.ui.config[parent_id][key].slider.setValue(slider_value)

    def on_update(
            self,
            parent_id: str,
            key: str,
            option: dict,
            value: any,
            type: str = None,
            hooks: bool = True
    ):
        """
        Event: on slider update

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option data
        :param value: New value
        :param type: Update from field type (slider, input, None=value)
        :param hooks: Run hooks
        """
        option['value'] = value
        self.apply(parent_id, key, option, type)

        # on update hooks
        if hooks:
            hook_name = "update.{}.{}".format(parent_id, key)
            if self.window.ui.has_hook(hook_name):
                hook = self.window.ui.get_hook(hook_name)
                try:
                    hook(key, value, type)
                except Exception as e:
                    self.window.core.debug.log(e)

    def get_value(
            self,
            parent_id: str,
            key: str,
            option: dict
    ) -> any:
        """
        Get slider value

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option data
        :return: Slider value (int or float)
        """
        is_integer = False
        multiplier = 1
        if "type" in option and option["type"] == "int":
            is_integer = True
        if "multiplier" in option:
            multiplier = option["multiplier"]
        value = self.window.ui.config[parent_id][key].slider.value()
        if is_integer:
            return round(int(value), 0)
        else:
            return value / multiplier



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.22 06:00:00                  #
# ================================================== #

import os


class Markdown:
    def __init__(self, window=None):
        """
        Markdown css controller

        :param window: Window instance
        """
        self.window = window
        self.css = {}  # external styles

    def update(self, force: bool = False):
        """
        Update markdown styles

        :param force: force theme change (manual trigger)
        """
        if force:
            self.window.controller.ui.store_state()  # store state before theme change

        if self.window.core.config.get('theme.markdown'):
            self.load()
        else:
            self.set_default()
        self.apply()

        if force:
            self.window.controller.ui.restore_state()  # restore state after theme change

    def set_default(self):
        """Set default markdown CSS"""
        self.css['markdown'] = self.get_default()

    def apply(self):
        """Apply CSS to markdown formatter"""
        self.window.ui.nodes['output_plain'].setStyleSheet(self.css['markdown'])
        if self.window.controller.chat.render.get_engine() == "legacy":
            self.window.ui.nodes['output'].setStyleSheet(self.css['markdown'])
            self.window.ui.nodes['output'].document().setDefaultStyleSheet(self.css['markdown'])
            self.window.ui.nodes['output'].document().setMarkdown(self.window.ui.nodes['output'].document().toMarkdown())
        elif self.window.controller.chat.render.get_engine() == "web":
            self.window.controller.chat.render.web_renderer.reload_css()
        self.window.controller.ctx.refresh_output()

    def get_web_css(self) -> str:
        """Get web CSS"""
        if "web" not in self.css:
            self.load()
        return self.css["web"]

    def clear(self):
        """Clear CSS of markdown formatter"""
        self.window.controller.chat.render.clear_all()
        self.window.controller.ctx.refresh()
        self.window.controller.ctx.refresh_output()
        self.window.controller.chat.render.end()

    def load(self):
        """Load markdown styles"""
        parents = ["markdown", "web"]

        for base_name in parents:
            theme = self.window.core.config.get('theme')
            name = str(base_name)
            color_name = str(base_name)
            if theme.startswith('light'):
                color_name += '.light'
            else:
                color_name += '.dark'
            paths = []
            paths.append(os.path.join(self.window.core.config.get_app_path(), 'data', 'css', name + '.css'))
            paths.append(os.path.join(self.window.core.config.get_app_path(), 'data', 'css', color_name + '.css'))
            paths.append(os.path.join(self.window.core.config.get_user_path(), 'css', name + '.css'))
            paths.append(os.path.join(self.window.core.config.get_user_path(), 'css', color_name + '.css'))
            content = ''
            for path in paths:
                if os.path.exists(path):
                    with open(path, 'r') as file:
                        content += file.read()
            try:
                self.css[base_name] = content.format(**os.environ)
            except KeyError as e:
                pass  # ignore missing env vars

    def get_default(self):
        """Set default markdown CSS"""
        colors = {
            "dark": {
                "a": "#fff",
                "msg-user": "#d9d9d9",
                "msg-bot": "#fff",
                "cmd": "#4d4d4d",
                "ts": "#d0d0d0",
                "pre-bg": "#202225",
                "pre": "#fff",
                "code": "#fff",
            },
            "light": {
                "a": "#000",
                "msg-user": "#444444",
                "msg-bot": "#000",
                "cmd": "#4d4d4d",
                "ts": "#4d4d4d",
                "pre-bg": "#e9e9e9",
                "pre": "#000",
                "code": "#000",
            }
        }

        theme = self.window.core.config.get('theme')
        styles = colors['dark']
        if theme.startswith('light'):
            styles = colors['light']

        return """
        a {{
            color: {a};
        }}
        .msg-user {{
            color: {msg-user} !important;
            white-space: pre-wrap;
            width: 100%;
            max-width: 100%;
        }}
        .msg-bot {{
            color: {msg-bot} !important;
            white-space: pre-wrap;
            width: 100%;
            max-width: 100%;
        }}
        .cmd {{
            color: {cmd};
        }}
        .ts {{
            color: {ts};
        }}
        .list {{
        }}
        pre {{
            color: {pre};
            background-color: {pre-bg};
            font-family: 'Lato';
            display: block;
        }}
        code {{
            color: {pre};
        }}""".format_map(styles)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 01:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QPushButton, QLabel, QVBoxLayout, QMenuBar, QTabWidget, QHBoxLayout, QSplitter, QWidget

from pygpt_net.ui.widget.dialog.base import BaseDialog
from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.ui.widget.option.combo import OptionCombo
from pygpt_net.ui.widget.textarea.editor import CodeEditor

from .browse import BrowseTab
from .ctx import CtxTab
from .files import FilesTab
from .web import WebTab

from pygpt_net.utils import trans

class DialogBuilder:
    def __init__(self, window=None):
        """
        Dialog builder

        :param window: Window instance
        """
        self.window = window
        self.menu_bar = None
        self.menu = {}
        self.actions = {}  # menu actions
        self.browse = BrowseTab(window)
        self.ctx = CtxTab(window)
        self.files = FilesTab(window)
        self.web = WebTab(window)

    def setup_menu(self) -> QMenuBar:
        """
        Setup dialog menu

        :return: QMenuBar
        """
        # create menu bar
        self.menu_bar = QMenuBar()
        self.menu["file"] = self.menu_bar.addMenu(trans("menu.file"))
        self.menu["config"] = self.menu_bar.addMenu(trans("menu.config"))

        self.actions["file.clear_log"] = QAction(QIcon(":/icons/close.svg"), trans("tool.indexer.menu.file.clear_log"))
        self.actions["file.clear_log"].triggered.connect(
            lambda: self.window.tools.get("indexer").clear_log()
        )

        self.actions["file.truncate_idx"] = QAction(QIcon(":/icons/delete.svg"), trans("tool.indexer.menu.file.remove_idx"))
        self.actions["file.truncate_idx"].triggered.connect(
            lambda: self.window.tools.get("indexer").truncate_idx()
        )

        self.actions["config.settings"] = QAction(QIcon(":/icons/settings.svg"), trans("tool.indexer.menu.config.settings"))
        self.actions["config.settings"].triggered.connect(
            lambda: self.window.tools.get("indexer").open_settings()
        )

        # add actions
        self.menu["file"].addAction(self.actions["file.clear_log"])
        self.menu["file"].addAction(self.actions["file.truncate_idx"])
        self.menu["config"].addAction(self.actions["config.settings"])
        return self.menu_bar

    def setup(self):
        """Setup dialog window"""
        browse = self.browse.setup()
        ctx = self.ctx.setup()
        files = self.files.setup()
        web = self.web.setup()

        # index select
        self.window.ui.nodes["tool.indexer.idx"] = OptionCombo(self.window, "tool.indexer", "idx", {
            "label": trans("tool.indexer.idx"),
            "keys": self.window.controller.config.placeholder.apply_by_id("idx"),
            "value": "base",
        })
        self.window.ui.add_hook("update.tool.indexer.idx", self.hook_idx_change)
        self.window.ui.nodes["tool.indexer.idx"].layout.setContentsMargins(0, 0, 0, 0)
        self.window.ui.nodes["tool.indexer.idx.label"] = QLabel(trans("tool.indexer.idx"))
        self.window.ui.nodes["tool.indexer.provider"] = HelpLabel(self.window.core.config.get("llama.idx.storage"))

        # set first index
        default_idx = None
        indexes = self.window.core.config.get("llama.idx.list")
        if indexes:
            default_idx = indexes[0]["id"]
        if default_idx:
            self.window.ui.nodes["tool.indexer.idx"].set_value(default_idx)

        idx_layout = QHBoxLayout()
        idx_layout.addWidget(self.window.ui.nodes["tool.indexer.provider"])
        idx_layout.addWidget(self.window.ui.nodes["tool.indexer.idx.label"], alignment=Qt.AlignRight)
        idx_layout.addWidget(self.window.ui.nodes["tool.indexer.idx"])

        self.window.ui.tabs['tool.indexer'] = QTabWidget(self.window)
        self.window.ui.tabs['tool.indexer'].addTab(files, trans("tool.indexer.tab.files"))
        self.window.ui.tabs['tool.indexer'].addTab(web, trans("tool.indexer.tab.web"))
        self.window.ui.tabs['tool.indexer'].addTab(ctx, trans("tool.indexer.tab.ctx"))
        self.window.ui.tabs['tool.indexer'].addTab(browse, trans("tool.indexer.tab.browser"))
        self.window.ui.tabs['tool.indexer'].currentChanged.connect(
            lambda: self.window.tools.get("indexer").on_tab_changed(self.window.ui.tabs['tool.indexer'].currentIndex())
        )

        self.window.ui.nodes["tool.indexer.status"] = CodeEditor(self.window)
        self.window.ui.nodes["tool.indexer.status"].setReadOnly(True)
        self.window.ui.nodes["tool.indexer.status"].setProperty('class', 'text-editor')
        self.window.ui.nodes["tool.indexer.btn.idx"] = QPushButton(trans("tool.indexer.idx.btn.add"))
        self.window.ui.nodes["tool.indexer.btn.idx"].clicked.connect(
            lambda: self.window.tools.get("indexer").index_data()
        )
        self.window.ui.nodes["tool.indexer.status.label"] = QLabel(trans("tool.indexer.status"))

        bottom_layout = QVBoxLayout()
        bottom_layout.addWidget(self.window.ui.nodes["tool.indexer.status.label"])
        bottom_layout.addWidget(self.window.ui.nodes["tool.indexer.status"])

        tabs_layout = QVBoxLayout()
        tabs_layout.setMenuBar(self.setup_menu())  # add menu bar
        tabs_layout.addLayout(idx_layout)
        tabs_layout.addWidget(self.window.ui.tabs['tool.indexer'])
        tabs_layout.addWidget(self.window.ui.nodes["tool.indexer.btn.idx"])

        tabs_widget = QWidget()
        tabs_widget.setLayout(tabs_layout)

        bottom_widget = QWidget()
        bottom_widget.setLayout(bottom_layout)

        self.window.ui.splitters['tool.indexer'] = QSplitter(Qt.Vertical)
        self.window.ui.splitters['tool.indexer'].addWidget(tabs_widget)
        self.window.ui.splitters['tool.indexer'].addWidget(bottom_widget)

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.splitters['tool.indexer'])
        layout.setContentsMargins(0, 0, 0, 0)

        # add dialog to the window - static dialog
        self.window.ui.dialog['tool.indexer'] = IndexerDialog(self.window)
        self.window.ui.dialog['tool.indexer'].setLayout(layout)
        self.window.ui.dialog['tool.indexer'].setWindowTitle(trans("tool.indexer.title"))

    def hook_idx_change(self, key, value, caller, *args, **kwargs):
        """
        Hook: on loader change

        :param key: Option key
        :param value: Option value
        :param caller: Caller
        :param args: Args
        :param kwargs: Kwargs
        """
        self.window.tools.get("indexer").set_current_idx(value, check=False)

class IndexerDialog(BaseDialog):
    def __init__(self, window=None):
        """
        Example dialog

        :param window: main window
        """
        super(IndexerDialog, self).__init__(window)
        self.window = window

    def closeEvent(self, event):
        """
        Close event

        :param event: close event
        """
        self.cleanup()
        super(IndexerDialog, self).closeEvent(event)

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key press event
        """
        if event.key() == Qt.Key_Escape:
            self.cleanup()
            self.close()  # close dialog when the Esc key is pressed.
        else:
            super(IndexerDialog, self).keyPressEvent(event)

    def cleanup(self):
        """Cleanup on dialog close"""
        self.window.tools.get("indexer").on_close()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.27 11:00:00                  #
# ================================================== #

import os

from pygpt_net.utils import trans


class Common:
    def __init__(self, window=None):
        """
        Theme common controller

        :param window: Window instance
        """
        self.window = window

    def get_extra_css(self, name: str) -> str:
        """
        Return custom css filename for specified theme

        :param name: theme name
        :return: custom css filename (e.g. style.dark.css)
        """
        # check per theme style css
        filename = 'style.css'
        if filename is not None:
            # per theme mode (light / dark)
            tmp = None
            if name.startswith('light_'):
                tmp = 'style.light.css'
            elif name.startswith('dark_'):
                tmp = 'style.dark.css'
            if tmp is not None:
                paths = []
                paths.append(os.path.join(self.window.core.config.get_user_path(), 'css', name + '.css'))
                paths.append(os.path.join(self.window.core.config.get_app_path(), 'data', 'css', name + '.css'))
                paths.append(os.path.join(self.window.core.config.get_user_path(), 'css', tmp))
                paths.append(os.path.join(self.window.core.config.get_app_path(), 'data', 'css', tmp))
                for path in paths:
                    if os.path.exists(path):
                        filename = tmp
                        break
        return filename

    def toggle_tooltips(self):
        """Toggle visibility of static tooltips"""
        nodes = [
            'tip.input.attachments',
            'tip.input.attachments.uploaded',
            'tip.output.tab.calendar',
            'tip.output.tab.draw',
            'tip.output.tab.files',
            'tip.output.tab.notepad',
            'tip.toolbox.assistants',
            'tip.toolbox.ctx',
            'tip.toolbox.indexes',
            'tip.toolbox.mode',
            'tip.toolbox.presets',
            'tip.toolbox.prompt',
        ]
        state = self.window.core.config.get('layout.tooltips')
        if state:
            for node in nodes:
                if node in self.window.ui.nodes:
                    self.window.ui.nodes[node].setVisible(True)
        else:
            for node in nodes:
                if node in self.window.ui.nodes:
                    self.window.ui.nodes[node].setVisible(False)

        self.window.ui.menu['theme.tooltips'].setChecked(state)

    def translate(self, theme: str) -> str:
        """
        Translate theme name

        :param theme: theme name
        :return: translated theme name
        """
        return theme \
            .replace('_', ' ').title() \
            .replace('Dark ', trans('theme.dark') + ': ') \
            .replace('Light ', trans('theme.light') + ': ')

    def get_style(self, element: str) -> str:
        """
        Return CSS style for element

        :param element: type of element
        :return: CSS style for element
        """
        # get font size
        if element == "font.chat.output":
            return 'font-size: {}px;'.format(self.window.core.config.get('font_size'))
        elif element == "font.chat.input":
            return 'font-size: {}px;'.format(self.window.core.config.get('font_size.input'))
        elif element == "font.ctx.list":
            return 'font-size: {}px;'.format(self.window.core.config.get('font_size.ctx'))
        elif element == "font.toolbox":
            return 'font-size: {}px;'.format(self.window.core.config.get('font_size.toolbox'))

    def get_themes_list(self) -> list:
        """
        Return a list of available themes

        :return: list of themes names
        """
        return [
            'dark_amber',
            'dark_blue',
            'dark_cyan',
            'dark_lightgreen',
            'dark_pink',
            'dark_purple',
            'dark_red',
            'dark_teal',
            'dark_yellow',
            'light_amber',
            'light_blue',
            'light_cyan',
            'light_cyan_500',
            'light_lightgreen',
            'light_pink',
            'light_purple',
            'light_red',
            'light_teal',
            'light_yellow',
        ]

    def get_windows_fix(self) -> str:
        """
        Return Windows checkbox button + radio button fix

        :return: stylesheet with fix
        """
        # abort if SVG is supported (no need to fix missing DLLs)
        if self.window.core.platforms.is_svg_supported():
            return ''

        filename = 'fix_windows.css'
        paths = []
        paths.append(os.path.join(self.window.core.config.get_app_path(), 'data', 'css', filename))
        paths.append(os.path.join(self.window.core.config.get_user_path(), 'css', filename))
        content = ''
        for path in paths:
            if os.path.exists(path):
                with open(path) as file:
                    content += file.read()
        return content



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.25 10:00:00                  #
# ================================================== #

import os

from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QWidget

from pygpt_net.utils import trans
import pygpt_net.icons_rc


class Actions:
    def __init__(self, window=None):
        """
        Filesystem actions

        :param window: Window instance
        """
        self.window = window

    def has_preview(self, path: str) -> bool:
        """
        Check if file has preview action

        :param path: path to file
        :return: True if file has preview
        """
        if os.path.isdir(path):
            return False
        return True

    def has_use(self, path: str) -> bool:
        """
        Check if file has preview action

        :param path: path to file
        :return: True if file has preview
        """
        return self.window.core.filesystem.types.is_image(path) or self.window.core.filesystem.types.is_video(path)

    def get_preview(self, parent: QWidget, path: str) -> list:
        """
        Get preview actions for context menu

        :param parent: explorer widget
        :param path: path to file
        :return: list of context menu actions
        """
        actions = []
        if self.window.core.filesystem.types.is_video(path) or self.window.core.filesystem.types.is_audio(path):
            action = QAction(
                QIcon(":/icons/video.svg"),
                trans('action.video.play'),
                parent,
            )
            action.triggered.connect(
                lambda: self.window.tools.get("player").play(path),
            )
            actions.append(action)
            action = QAction(
                QIcon(":/icons/hearing.svg"),
                trans('action.video.transcribe'),
                parent,
            )
            action.triggered.connect(
                lambda: self.window.tools.get("transcriber").from_file(path),
            )
            actions.append(action)
        elif self.window.core.filesystem.types.is_image(path):
            action = QAction(
                QIcon(":/icons/image.svg"),
                trans('action.preview'),
                parent,
            )
            action.triggered.connect(
                lambda: self.window.tools.get("viewer").open_preview(path),
            )
            actions.append(action)
        else:
            extra_excluded = ["pdf", "docx", "doc", "xlsx", "xls", "pptx", "ppt"]
            ext = os.path.splitext(path)[1][1:].lower()
            if ext not in self.window.core.filesystem.types.get_excluded_extensions() + extra_excluded:
                action = QAction(
                    QIcon(":/icons/edit.svg"),
                    trans('action.edit'),
                    parent,
                )
                action.triggered.connect(
                    lambda: self.window.tools.get("editor").open(path),
                )
                actions.append(action)
        return actions

    def get_use(self, parent: QWidget, path: str) -> list:
        """
        Get use actions for context menu

        :param parent: explorer widget
        :param path: path to file
        :return: list of context menu actions
        """
        actions = []
        if self.window.core.filesystem.types.is_image(path):
            action = QAction(
                QIcon(":/icons/brush.svg"),
                trans('action.use.image'),
                parent,
            )
            action.triggered.connect(
                lambda: self.window.controller.painter.open_external(path),
            )
            actions.append(action)
        return actions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.17 13:00:00                  #
# ================================================== #

import copy
import json

from pygpt_net.item.ctx import CtxItem


class Command:
    def __init__(self, window=None):
        """
        Commands core

        :param window: Window instance
        """
        self.window = window

    def append_syntax(self, data: dict) -> str:
        """
        Append command syntax to the system prompt

        :param data: event data
        :return: prompt with appended syntax
        """
        prompt = data['prompt']
        cmd_prompt = self.window.core.prompt.get('cmd')
        extra = ""
        schema = self.extract_syntax(data['cmd'])
        if schema:
            if self.window.core.config.get('mode') == "assistant":
                extra = self.window.core.prompt.get('cmd.extra.assistants')  # Assistants API env fix
            else:
                extra = self.window.core.prompt.get('cmd.extra')
        if prompt.strip() != "":
            prompt += "\n\n"
        prompt += cmd_prompt.strip().replace("{extra}", extra).replace("{schema}", schema)
        return prompt

    def extract_syntax(self, cmds: list) -> str:
        """
        Extract syntax from commands

        :param cmds: commands list
        :return: JSON string with commands usage syntax
        """
        data = {}
        cmds = copy.deepcopy(cmds)  # make copy to prevent changes in original data
        self.window.core.ctx.current_cmd = copy.deepcopy(cmds)  # for debug purposes

        for cmd in cmds:
            if "cmd" in cmd and "instruction" in cmd:
                cmd_name = cmd["cmd"]
                data[cmd_name] = {
                    "help": cmd["instruction"],
                }
                if "params" in cmd and len(cmd["params"]) > 0:
                    data[cmd_name]["params"] = {}
                    for param in cmd["params"]:
                        try:
                            if isinstance(param, dict):
                                if "name" in param:
                                    # assign param
                                    key = param["name"]
                                    del param["name"]
                                    data[cmd_name]["params"][key] = param

                                    # remove required, leave only optional
                                    if "required" in data[cmd_name]["params"][key]:
                                        if data[cmd_name]["params"][key]["required"] is False:
                                            data[cmd_name]["params"][key]["optional"] = True
                                        del data[cmd_name]["params"][key]["required"]

                                    # remove type if str (default)
                                    if ("type" in data[cmd_name]["params"][key]
                                            and data[cmd_name]["params"][key]["type"] == "str"):
                                        del data[cmd_name]["params"][key]["type"]

                                    # remove description and move to help
                                    if "description" in data[cmd_name]["params"][key]:
                                        data[cmd_name]["params"][key]["help"] = data[cmd_name]["params"][key]["description"]
                                        del data[cmd_name]["params"][key]["description"]

                        except Exception as e:
                            pass

        self.window.core.ctx.current_cmd_schema = data  # for debug
        return json.dumps(data)  # pack, return JSON string without indent and formatting

    def extract_cmds(self, text: str) -> list:
        """
        Extract commands from text

        :param text: text to extract commands from
        :return: list of commands (dict)
        """
        cmds = []
        try:
            chunks = text.split('~###~')
            for chunk in chunks:
                cmd = self.extract_cmd(chunk)  # extract JSON string to dict
                if cmd is not None:
                    cmds.append(cmd)  # cmd = dict
        except Exception as e:
            # do nothing
            pass
        return cmds

    def extract_cmd(self, chunk: str) -> dict or None:
        """
        Extract command from text chunk (JSON string)

        :param chunk: text chunk (JSON command string)
        :return: command dict
        """
        cmd = None
        chunk = chunk.strip()
        if chunk and chunk.startswith('{') and chunk.endswith('}'):
            try:
                # syntax1: {"read_file": {"path": ["my_cars.txt"]}}
                # syntax2: {"cmd": "read_file", "params": {"path": ["my_cars.txt"]}}
                cmd = json.loads(chunk)

                # if the first key is not "cmd", then try to convert from incorrect syntax into "cmd" syntax:
                if "cmd" not in cmd:
                    if len(cmd) == 1:
                        for key in cmd:
                            if "params" in cmd[key]:
                                cmd = {
                                    "cmd": key,
                                    "params": cmd[key]["params"]
                                }
                            else:
                                cmd = {
                                    "cmd": key,
                                    "params": cmd[key]
                                }
            except json.JSONDecodeError as e:
                # do nothing
                pass
        return cmd

    def unpack_tool_calls(self, tool_calls: list) -> list:
        """
        Unpack tool calls from OpenAI response

        :param tool_calls: tool calls list
        :return: parsed tool calls list
        """
        parsed = []
        for tool_call in tool_calls:
            try:
                parsed.append(
                    {
                        "id": tool_call.id,
                        "type": "function",
                        "function": {
                            "name": tool_call.function.name,
                            "arguments": json.loads(tool_call.function.arguments)
                        }
                    }
                )
            except Exception as e:
                self.window.core.debug.log(e)
                print("Error parsing tool call: " + str(e))
        return parsed

    def unpack_tool_calls_chunks(self, ctx: CtxItem, tool_calls: list):
        """
        Handle / unpack tool calls

        :param ctx: context
        :param tool_calls: tool calls
        """
        tmp_calls = []
        for tool_call in tool_calls:
            try:
                if "function" not in tool_call:
                    continue
                if "arguments" not in tool_call["function"]:
                    continue
                tool_call["function"]["arguments"] = json.loads(
                    tool_call["function"]["arguments"]
                )
                tmp_calls.append(tool_call)
            except Exception as e:
                self.window.core.debug.log(e)
                print("Error parsing tool call JSON arguments: ", tool_call["function"]["arguments"])
        ctx.tool_calls = tmp_calls

    def tool_call_to_cmd(self, tool_call: dict) -> dict:
        """
        Convert tool call to command

        :param tool_call: tool call
        :return: command
        """
        return {
            "cmd": tool_call["function"]["name"],
            "params": tool_call["function"]["arguments"]
        }

    def tool_calls_to_cmds(self, tool_calls: list) -> list:
        """
        Convert tool calls to commands

        :param tool_calls: tool calls
        :return: commands
        """
        cmds = []
        for tool_call in tool_calls:
            cmds.append(self.tool_call_to_cmd(tool_call))
        return cmds

    def pack_cmds(self, cmds: list) -> str:
        """
        Pack commands to string

        :param cmds: commands
        :return: packed commands string
        """
        packed = ""
        for cmd in cmds:
            packed += "~###~" + json.dumps(cmd) + "~###~"
        return packed

    def append_tool_calls(self, ctx: CtxItem):
        """
        Append tool calls as CMD to context output

        :param ctx: context item
        """
        cmds = self.tool_calls_to_cmds(ctx.tool_calls)
        cmds_str = self.pack_cmds(cmds)
        if ctx.output is None:
            ctx.output = ""
        ctx.output += cmds_str

    def get_tool_calls_outputs(self, ctx: CtxItem) -> list:
        """
        Prepare and get tool calls outputs

        :param ctx: context item
        :return: list of tool calls outputs
        """
        outputs = []
        idx_outputs = {}
        for tool_call in ctx.tool_calls:
            call_id = tool_call["id"]
            idx_outputs[call_id] = ""
        try:
            responses = json.loads(ctx.input)  # response is JSON string in input
            for tool_call in ctx.tool_calls:
                call_id = tool_call["id"]
                for response in responses:
                    if "request" in response:
                        if "cmd" in response["request"]:
                            func_name = response["request"]["cmd"]
                            if tool_call["function"]["name"] == func_name:
                                idx_outputs[call_id] = response["result"]

        except Exception as e:
            self.window.core.debug.log(e)

        for id in idx_outputs:
            data = {
                "tool_call_id": id,
                "output": idx_outputs[id],
            }
            outputs.append(data)

        if not isinstance(ctx.extra, dict):
            ctx.extra = {}
        ctx.extra["tool_calls_outputs"] = outputs
        return outputs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 01:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtGui import QCursor
from typing_extensions import override

from PySide6.QtWidgets import QApplication, QMenu, QHeaderView

from pygpt_net.ui.dialog.db import DataBrowser
from pygpt_net.ui.widget.lists.db import DatabaseList
from pygpt_net.utils import trans


class IdxBrowseList(DatabaseList):

    def adjustColumns(self):
        """Adjust columns width"""
        last_column = self.model().columnCount() - 1
        self.horizontalHeader().setSectionResizeMode(last_column, QHeaderView.Stretch)

    def create_context_menu(self, parent):
        """
        Create context menu

        :param parent: Parent
        """

        def copy_to_clipboard():
            index = parent.currentIndex()
            if index.isValid():
                value = index.sibling(index.row(), 1).data(
                    Qt.DisplayRole)
                QApplication.clipboard().setText(value)

        def delete_row():
            index = parent.currentIndex()
            if index.isValid():
                id = index.sibling(index.row(), 0).data(
                    Qt.DisplayRole)
                self.window.tools.get("indexer").delete_db_idx(int(id))

        menu = QMenu()
        copy_action = menu.addAction(trans("tool.indexer.db.copy"))
        delete_action = menu.addAction(trans("tool.indexer.db.remove"))
        copy_action.triggered.connect(copy_to_clipboard)
        delete_action.triggered.connect(delete_row)
        menu.exec_(QCursor.pos())

class IdxBrowser(DataBrowser):
    @override
    def get_tables(self) -> dict:
        """
        Get tables configuration

        :return: Tables dictionary
        """
        data = self.window.tools.get("indexer").get_tables()
        tables = {k: v for k, v in data.items() if k in [
            "idx_ctx", "idx_file", "idx_external"]}
        return tables

    @override
    def get_filters(self):
        return {
            "idx": self.window.tools.get("indexer").current_idx,
            "store": self.window.core.config.get("llama.idx.storage"),
        }

    @override
    def get_table_name(self, selected_text):
        keys = self.get_tables().keys()
        mapping = {
            "idx_file": "Files",
            "idx_external": "Web",
            "idx_ctx": "Context",
        }
        for key in keys:
            if mapping[key] == selected_text:
                return key

    @override
    def get_table_names(self):
        return [
            "Files",
            "Web",
            "Context",
        ]

    @override
    def get_default_table(self):
        """
        Get default table

        :return: Default table name
        """
        return "idx_file"

    @override
    def set_list_widget(self):
        self.window.ui.nodes["tool.indexer.browse"] = IdxBrowseList(self.window)

    @override
    def get_list_widget(self):
        return self.window.ui.nodes["tool.indexer.browse"]

    @override
    def is_editable(self) -> bool:
        return False

    @override
    def is_inline(self) -> bool:
        return True